@@ -45,7 +45,7 @@ from xonsh.tools import (
     XonshError,
     XonshCalledProcessError,
 )
-from xonsh.lazyimps import pty, termios
+from xonsh.lazyimps import pty, termios, fcntl
 from xonsh.commands_cache import CommandsCache
 from xonsh.events import events
 
@@ -377,18 +377,18 @@ class SubprocSpec:
     executed.
     """
 
-    kwnames = ("stdin", "stdout", "stderr", "universal_newlines")
+    kwnames = ("stdin", "stdout", "stderr", "universal_newlines", "close_fds")
 
     def __init__(
         self,
         cmd,
-        *,
         cls=subprocess.Popen,
         stdin=None,
         stdout=None,
         stderr=None,
         universal_newlines=False,
-        captured=False
+        close_fds=False,
+        captured=False,
     ):
         """
         Parameters
@@ -405,6 +405,9 @@ class SubprocSpec:
             Popen file descriptor or flag for stderr.
         universal_newlines : bool
             Whether or not to use universal newlines.
+        close_fds : bool
+            Whether or not to close the file descriptiors when the
+            process exits.
         captured : bool or str, optional
             The flag for if the subprocess is captured, may be one of:
             False for $[], 'stdout' for $(), 'hiddenobject' for ![], or
@@ -445,6 +448,7 @@ class SubprocSpec:
         self.stdout = stdout
         self.stderr = stderr
         self.universal_newlines = universal_newlines
+        self.close_fds = close_fds
         self.captured = captured
         # pure attrs
         self.args = list(cmd)
@@ -531,6 +535,8 @@ class SubprocSpec:
 
     def run(self, *, pipeline_group=None):
         """Launches the subprocess and returns the object."""
+        event_name = self._cmd_event_name()
+        self._pre_run_event_fire(event_name)
         kwargs = {n: getattr(self, n) for n in self.kwnames}
         self.prep_env(kwargs)
         self.prep_preexec_fn(kwargs, pipeline_group=pipeline_group)
@@ -545,6 +551,7 @@ class SubprocSpec:
         p.last_in_pipeline = self.last_in_pipeline
         p.captured_stdout = self.captured_stdout
         p.captured_stderr = self.captured_stderr
+        self._post_run_event_fire(event_name, p)
         return p
 
     def _run_binary(self, kwargs):
@@ -600,6 +607,26 @@ class SubprocSpec:
         for i in range(len(cmd)):
             cmd[i] = cmd[i].replace("\0", "\\0")
 
+    def _cmd_event_name(self):
+        if callable(self.alias):
+            return getattr(self.alias, "__name__", repr(self.alias))
+        elif self.binary_loc is None:
+            return "<not-found>"
+        else:
+            return os.path.basename(self.binary_loc)
+
+    def _pre_run_event_fire(self, name):
+        event_name = "on_pre_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self)
+
+    def _post_run_event_fire(self, name, proc):
+        event_name = "on_post_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self, proc=proc)
+
     #
     # Building methods
     #
@@ -744,6 +771,17 @@ def _safe_pipe_properties(fd, use_tty=False):
     props = termios.tcgetattr(fd)
     props[1] = props[1] & (~termios.ONLCR) | termios.ONLRET
     termios.tcsetattr(fd, termios.TCSANOW, props)
+    # newly created PTYs have a stardard size (24x80), set size to the same size
+    # than the current terminal
+    winsize = None
+    if sys.stdin.isatty():
+        winsize = fcntl.ioctl(sys.stdin.fileno(), termios.TIOCGWINSZ, b"0000")
+    elif sys.stdout.isatty():
+        winsize = fcntl.ioctl(sys.stdout.fileno(), termios.TIOCGWINSZ, b"0000")
+    elif sys.stderr.isatty():
+        winsize = fcntl.ioctl(sys.stderr.fileno(), termios.TIOCGWINSZ, b"0000")
+    if winsize is not None:
+        fcntl.ioctl(fd, termios.TIOCSWINSZ, winsize)
 
 
 def _update_last_spec(last):
@@ -1070,6 +1108,8 @@ def convert_macro_arg(raw_arg, kind, glbs, locs, *, name="<arg>", macroname="<ma
         if locs is not None:
             ctx |= set(locs.keys())
         mode = mode or "eval"
+        if mode != "eval" and not raw_arg.endswith("\n"):
+            raw_arg += "\n"
         arg = execer.parse(raw_arg, ctx, mode=mode, filename=filename)
     elif kind is types.CodeType or kind is compile:  # NOQA
         mode = mode or "eval"
@@ -1295,7 +1335,7 @@ class XonshSession:
     def __init__(self, execer=None, ctx=None):
         """
         Parameters
-        ---------
+        ----------
         execer : Execer, optional
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
@@ -1308,7 +1348,7 @@ class XonshSession:
         """Loads the session with default values.
 
         Parameters
-        ---------
+        ----------
         execer : Execer, optional
             Xonsh execution object, may be None to start
         ctx : Mapping, optional
